@@ -27,7 +27,11 @@
 {% endmacro %}
 
 
-{% macro singlestore__create_table_as(temporary, relation, sql) -%}
+{% macro check_for_constraint(primary_key) -%}
+    {% do return(adapter.check_for_constraint(raw_model_constraints=model['constraints'], raw_column_constraints=model['columns'], primary_key=primary_key)) -%}
+{% endmacro %}
+
+{% macro singlestore__create_table_as(temporary, relation, compiled_code, language='sql') -%}
     {%- set sql_header = config.get('sql_header', none) -%}
     {%- set primary_key = config.get('primary_key', []) -%} {# PRIMARY KEY (primary_key) #}
     {%- set sort_key = config.get('sort_key', []) -%} {# SORT KEY (sort_key) #}
@@ -35,9 +39,19 @@
     {%- set unique_table_key = config.get('unique_table_key', []) -%} {# UNIQUE KEY (unique_table_key) #}
     {%- set charset = config.get('charset', none) -%} {# CHARACTER SET charset #}
     {%- set collation = config.get('collation', none) -%} {# COLLATE collation #}
+    {%- set contract_config = config.get('contract') -%}
 
     {%- set create_definition_list = [] %}
-    {% if primary_key | length -%}
+    {%- set contract_defined_primary = False %}
+    {%- set contract_defined_unique = False %}
+    {%- set undefined_shard_key = True %}
+
+    {% if contract_config.enforced %}
+        {% set contract_defined_primary = check_for_constraint(True) -%}
+        {% set contract_defined_unique = check_for_constraint(False) -%}
+    {% endif -%}
+
+    {% if primary_key | length and (not contract_config.enforced or (contract_config.enforced and not contract_defined_primary)) %}
         {% set quoted = [] -%}
             {%- for col in primary_key -%}
                 {%- do quoted.append(adapter.quote(col)) -%}
@@ -56,15 +70,16 @@
             {%- for col in shard_key -%}
                 {%- do quoted.append(adapter.quote(col)) -%}
             {%- endfor %}
+        {% set undefined_shard_key = False -%}
         {% do create_definition_list.append('SHARD KEY ({})'.format(", ".join(quoted))) -%}
-    {% elif unique_table_key | length -%}
+    {% elif unique_table_key | length and (not contract_config.enforced or (contract_config.enforced and not contract_defined_unique)) -%}
         {% set quoted = [] -%}
             {%- for col in unique_table_key -%}
                 {%- do quoted.append(adapter.quote(col)) -%}
             {%- endfor %}
         {% do create_definition_list.append('SHARD KEY ({})'.format(", ".join(quoted))) -%}
     {% endif -%}
-    {% if unique_table_key | length -%}
+    {% if unique_table_key | length and (not contract_config.enforced or (contract_config.enforced and not contract_defined_unique)) -%}
         {% set quoted = [] -%}
             {%- for col in unique_table_key -%}
                 {%- do quoted.append(adapter.quote(col)) -%}
@@ -73,9 +88,13 @@
     {% endif -%}
 
     {% if create_definition_list | length -%}
-        {% set create_definition_str = '(' + create_definition_list|join(", ") + ')' -%}
-    {% else -%}
-        {% set create_definition_str = '(SHARD KEY ())' -%}
+        {% set create_definition_str = create_definition_list|join(", ") -%}
+    {% elif not contract_defined_primary and not contract_defined_unique -%}
+        {% set create_definition_str = 'SHARD KEY ()' -%}
+    {% endif -%}
+
+    {% if not contract_config.enforced -%}
+        {% set create_definition_str = '(' + create_definition_str + ')' -%}
     {% endif -%}
 
     {%- set charset_definition_str = ' ' %}
@@ -97,9 +116,17 @@
     {% endif -%}
 
     create {{ storage_type }} table
-        {{ relation.include(database=True) }} {{create_definition_str }} {{ charset_definition_str }}
+        {{ relation.include(database=True) }}
+    {%- if contract_config.enforced -%}
+      {{ get_assert_columns_equivalent(sql) }}
+      {{ singlestore__get_table_columns_and_constraints(create_definition_str, undefined_shard_key=undefined_shard_key) }}
+      {% set compiled_code = get_select_subquery(compiled_code) %}
+    {% else -%}
+        {{ create_definition_str }}
+    {% endif %}
+    {{ charset_definition_str }}
     as
-        {{ sql }}
+        {{ compiled_code }}
 {% endmacro %}
 
 
@@ -139,13 +166,6 @@
 {% endmacro %}
 
 
-{% macro singlestore__drop_schema(relation) -%}
-    {%- call statement('drop_database') -%}
-        drop database if exists {{ relation.database }}
-    {% endcall %}
-{% endmacro %}
-
-
 {% macro singlestore__drop_relation(relation) -%}
     {% call statement('drop_relation', auto_begin=False) -%}
         drop {{ relation.type }} if exists {{ relation.include(database=True) }}
@@ -178,7 +198,7 @@
     {% else -%}
         {{ '' }}
     {% endif -%}
-{% endmacro -%}}
+{% endmacro -%}
 
 
 {% macro singlestore__rename_relation(from_relation, to_relation) -%}
@@ -212,15 +232,14 @@
     {% endcall %}
 {% endmacro %}
 
-
-{% macro singlestore__current_timestamp() -%}
-    current_timestamp()
-{%- endmacro %}
-
-
+ 
 {% macro singlestore__create_view_as(relation, sql) -%}
     {%- set sql_header = config.get('sql_header', none) -%}
     {{ sql_header if sql_header is not none }}
+    {%- set contract_config = config.get('contract') -%}
+    {%- if contract_config.enforced -%}
+        {{ get_assert_columns_equivalent(sql) }}
+    {%- endif %}
     create view {{ relation }} as
         {{ sql }}
 {%- endmacro %}
