@@ -1,4 +1,3 @@
-
 {% materialization snapshot, adapter='singlestore' %}
   {%- set config = model['config'] -%}
 
@@ -6,6 +5,8 @@
 
   {%- set strategy_name = config.get('strategy') -%}
   {%- set unique_key = config.get('unique_key') %}
+  -- grab current tables grants config for comparision later on
+  {%- set grant_config = config.get('grants') -%}
 
   {% call statement('create_database') %}
      create database if not exists {{ model.database }}
@@ -92,6 +93,9 @@
 
   {% endif %}
 
+  {% set should_revoke = should_revoke(target_relation_exists, full_refresh_mode=False) %}
+  {% do apply_grants(target_relation, grant_config, should_revoke=should_revoke) %}
+
   {% do persist_docs(target_relation, model) %}
 
   {{ run_hooks(post_hooks, inside_transaction=True) }}
